/**
 * Copyright (c) 2010 Alois Glomann gnalff@lucubrators.net
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package net.lucubrators.honeycomb.core.controller;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.lucubrators.honeycomb.core.messages.Messages;
import net.lucubrators.honeycomb.core.shared.Model;
import net.lucubrators.honeycomb.core.view.View;

/**
 * This is a wrapper class storing all results of a controller. By preserving
 * the order of the results
 * 
 * @author aglomann
 * 
 */
public final class ControllerResult {

	public static final String					REQUEST_ATTRIBUTE_KEY	= "X-Honeycomby-" + ControllerResult.class;

	private final View							view;
	private Map<String, List<ControllerResult>>	marks;
	private Model								model;
	private FormObjectHolder					formObjectHolder;
	private Messages							messages;

	/**
	 * Construcotr
	 * 
	 * @param view
	 *            to be rendered
	 * @param model
	 *            the implicit model of the engine
	 */
	public ControllerResult(final View view, final Model model) {
		if (null == view || null == model) {
			throw new NullPointerException("None of the arguments must be null.");
		}
		this.view = view;
		this.model = model;
		this.marks = new HashMap<String, List<ControllerResult>>();
		this.messages = new Messages();
	}

	/**
	 * constructor
	 * 
	 * @param view
	 *            to be rendered
	 * @param model
	 *            the implicit model of the engine
	 * @param formObjectHolder
	 *            {@link FormObjectHolder} holding all informatio about
	 *            validiation and other errors related with the formobject
	 */
	public ControllerResult(final View view, final Model model, final FormObjectHolder formObjectHolder) {
		this(view, model);
		this.formObjectHolder = formObjectHolder;
	}

	/**
	 * set submarks, which are in direct relations to the ordering of combs
	 * 
	 * @param marks
	 *            a map representing all submarks
	 */
	public void setMarks(final Map<String, List<ControllerResult>> marks) {
		this.marks = marks;
	}

	/**
	 * @return the view to be rendered
	 */
	public View getView() {
		return view;
	}

	/**
	 * @return the model to be included
	 */
	public Model getModel() {
		return model;
	}

	/**
	 * @return a list of submarks of this result
	 */
	public Map<String, List<ControllerResult>> getMarks() {
		return marks;
	}

	/**
	 * 
	 * @return the applied {@link FormObjectHolder} or null
	 */
	public FormObjectHolder getFormObjectHolder() {
		return formObjectHolder;
	}

	public void setMessages(final Messages messages) {
		this.messages = messages;
	}

	public Messages getMessages() {
		return messages;
	}

}
