/**
 * Copyright (c) 2010 Alois Glomann gnalff@lucubrators.net
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package net.lucubrators.honeycomb.core.engine;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map.Entry;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import net.lucubrators.honeycomb.core.Comb;
import net.lucubrators.honeycomb.core.bind.BinderProvider;
import net.lucubrators.honeycomb.core.constraint.ConstraintHolder;
import net.lucubrators.honeycomb.core.controller.ControllerResult;
import net.lucubrators.honeycomb.core.controller.FormObjectHolder;
import net.lucubrators.honeycomb.core.controller.FormObjectProvider;
import net.lucubrators.honeycomb.core.reflect.exceptions.CombInvocationException;
import net.lucubrators.honeycomb.core.util.ControllerResultUtil;
import net.lucubrators.honeycomb.core.util.RequestMethod;
import net.lucubrators.honeycomb.core.view.NoOpView;
import net.lucubrators.honeycomb.core.view.RequestDispatcherViewContextFactory;
import net.lucubrators.honeycomb.core.view.View;
import net.lucubrators.honeycomb.core.view.ViewContext;
import net.lucubrators.honeycomb.core.view.ViewContextFactory;

public class HoneycombEngine {

	private static Logger			log	= LoggerFactory.getLogger(HoneycombEngine.class);

	private ConstraintInvoker		constraintInvoker;

	private ControllerInvoker		controllerInvoker;

	private SubmittingCombResolver	submittingCombResolver;

	private BinderMethodInvoker		binderMethodInvoker;

	private BinderProvider			binderProvider;

	private BindablesFactory		bindablesFactory;

	private ViewContextFactory		viewContextFactory;

	public HoneycombEngine() {
		this(new RequestDispatcherViewContextFactory());
	}

	protected HoneycombEngine(ViewContextFactory viewContextFactory) {
		this.viewContextFactory = viewContextFactory;
	}

	public void start(HttpServletRequest request, HttpServletResponse response, Comb comb) throws CombInvocationException {
		RequestMethod resolveMethod = RequestMethod.resolveMethod(request);
		ControllerResult result = null;
		switch (resolveMethod) {
		case GET:
			log.trace("Doing a get processing");
			result = walk(request, response, comb, null, null);
			break;
		case POST:
			Comb findSubmittingComb = submittingCombResolver.findSubmittingComb(comb, request);
			if (findSubmittingComb == null) {
				log.debug("Doing a get since i found now comb which should handle the post");
				result = walk(request, response, comb, null, null);
			} else {
				Bindables bindables = bindablesFactory.createBindables(request, response, findSubmittingComb);
				boolean doConstraintsBlock = doConstraintsBlock(findSubmittingComb.getConstraintHolders(), request, response, bindables);
				if (doConstraintsBlock) {
					log.debug("Constraint blocked submitting comb. Processing a get request but filtering out submitting comb");
					result = walk(request, response, comb, findSubmittingComb, new ControllerResult(NoOpView.INSTANCE, bindables.getModel()));
				} else {
					ControllerResult executeControllerinPostMode = executeController(findSubmittingComb.getController(), request, response, bindables,
							RequestMethod.POST);
					if (executeControllerinPostMode == null) {
						log.debug("Submitting comb returned null. Quitting here");
						result = null;
					} else {
						log.debug("Submitting comb returned a view. Doing get processing on other combs");
						result = walk(request, response, comb, findSubmittingComb, executeControllerinPostMode);
					}
				}
			}
		}
		if (result == null) {
			return;
		}
		ControllerResultUtil.setCurrentResult(request, result);
		ViewContext viewContext = viewContextFactory.create(request, response, result);
		result.getView().render(viewContext);
	}

	private ControllerResult walk(HttpServletRequest request, HttpServletResponse response, Comb comb, Comb submitted,
			ControllerResult submittedControllerResult) throws CombInvocationException {

		ControllerResult controllerResult = null;
		if (comb == submitted) {
			controllerResult = submittedControllerResult;
		} else {
			Bindables bindables = bindablesFactory.createBindables(request, response, comb);
			boolean contraintsBlock = doConstraintsBlock(comb.getConstraintHolders(), request, response, bindables);
			if (contraintsBlock) {
				return new ControllerResult(NoOpView.INSTANCE, bindables.getModel());
			}
			controllerResult = executeController(comb.getController(), request, response, bindables, RequestMethod.GET);
		}
		if (controllerResult == null) {
			return null;
		}
		if (controllerResult.getView() == NoOpView.INSTANCE) {
			return controllerResult;
		}
		Set<Entry<String, List<Comb>>> entrySet = comb.getMarks().entrySet();
		HashMap<String, List<ControllerResult>> marks = new HashMap<String, List<ControllerResult>>();
		for (Entry<String, List<Comb>> entry : entrySet) {
			List<Comb> subCombs = entry.getValue();
			ArrayList<ControllerResult> innerResults = new ArrayList<ControllerResult>();
			for (Comb subComb : subCombs) {
				ControllerResult walk = walk(request, response, subComb, submitted, submittedControllerResult);
				if (walk == null) {
					return null;
				}
				innerResults.add(walk);
			}
			marks.put(entry.getKey(), innerResults);
		}
		controllerResult.setMarks(marks);
		return controllerResult;
	}

	private boolean doConstraintsBlock(List<ConstraintHolder> constraintHolders, HttpServletRequest request, HttpServletResponse response, Bindables bindables)
			throws CombInvocationException {
		for (ConstraintHolder constraintHolder : constraintHolders) {
			bindables.setConstraintHolder(constraintHolder);
			Object invokeConstraint = constraintInvoker.invokeConstraint(constraintHolder.getConstraint(), bindables);
			bindables.setConstraintHolder(null);
			if (invokeConstraint == null || !Boolean.valueOf(invokeConstraint.toString())) {
				return true;
			}
		}
		return false;
	}

	private ControllerResult executeController(Object controller, HttpServletRequest request, HttpServletResponse response, Bindables bindables,
			RequestMethod method) throws CombInvocationException {
		FormObjectHolder formObjectHolder = null;
		if (controllerInvoker.hasMethodAnnotatedWith(controller, FormObjectProvider.class)) {
			formObjectHolder = controllerInvoker.getFormObject(controller, bindables);
			if (formObjectHolder == null) {
				return null;
			}
			bindables.setFormObjectHolder(formObjectHolder);
			bindables.getModel().put(formObjectHolder.getName(), formObjectHolder.getObject());
			if (RequestMethod.POST == method) {
				Object binder = binderProvider.getBinder(formObjectHolder.getBinder());
				binderMethodInvoker.invokeBinder(binder, bindables);
			}
		}
		Object controllerResponse = null;
		switch (method) {
		case GET:
			controllerResponse = controllerInvoker.doGet(controller, bindables);
			break;
		case POST:
			controllerResponse = controllerInvoker.doPost(controller, bindables);
			break;
		}
		if (controllerResponse == null || !(controllerResponse instanceof View)) {
			return null;
		}
		ControllerResult controllerResult = new ControllerResult((View) controllerResponse, bindables.getModel(), formObjectHolder);
		controllerResult.setMessages(bindables.getMessages());
		return controllerResult;
	}

	public void setBindablesFactory(BindablesFactory bindablesFactory) {
		this.bindablesFactory = bindablesFactory;
	}

	public void setConstraintInvoker(ConstraintInvoker constraintInvoker) {
		this.constraintInvoker = constraintInvoker;
	}

	public void setControllerInvoker(ControllerInvoker controllerInoker) {
		this.controllerInvoker = controllerInoker;
	}

	public void setSubmittingCombResolver(SubmittingCombResolver submittingCombResolver) {
		this.submittingCombResolver = submittingCombResolver;
	}

	public void setBinderMethodInvoker(BinderMethodInvoker binderMethodInvoker) {
		this.binderMethodInvoker = binderMethodInvoker;
	}

	public void setBinderProvider(BinderProvider binderProvider) {
		this.binderProvider = binderProvider;
	}

}
